use nom::branch::alt;
use nom::bytes::complete::is_not;
use nom::bytes::complete::tag_no_case;
use nom::character::complete::anychar;
use nom::character::complete::line_ending;
use nom::character::complete::space1;
use nom::combinator::eof;
use nom::multi::many_till;
use nom::sequence::tuple;

use super::OrgSource;
use crate::error::Res;
use crate::parser::util::start_of_line;

#[cfg_attr(feature = "tracing", tracing::instrument(ret, level = "debug"))]
pub fn scan_for_setup_file<'s>(input: OrgSource<'s>) -> Res<OrgSource<'s>, OrgSource<'s>> {
    let (remaining, setup) = many_till(anychar, setup_file)(input)
        .map(|(remaining, (_, setup_file))| (remaining, setup_file))?;
    Ok((remaining, setup))
}

fn setup_file<'s>(input: OrgSource<'s>) -> Res<OrgSource<'s>, OrgSource<'s>> {
    let (remaining, (_, _, _, setup_file, _)) = tuple((
        start_of_line,
        tag_no_case("#+SETUPFILE:"),
        space1,
        is_not("\r\n"),
        alt((line_ending, eof)),
    ))(input)?;
    Ok((remaining, setup_file))
}
